==> infra-4.3.1-sources.jar: com/liveperson/infra/utils/TextCrawler.java <==
package com.liveperson.infra.utils;

import android.os.AsyncTask;
import android.support.annotation.NonNull;
import android.text.TextUtils;
import android.util.Patterns;

import com.liveperson.infra.R;
import com.liveperson.infra.configuration.Configuration;
import com.liveperson.infra.log.LPMobileLog;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Created by maayank on 17/11/2016.
 * a util class for link preview feature (and for structure content)
 * receives a URL and parse the open graph (and other relevant) tags from it
 * it uses a callback to return the response
 * we use AsyncTask with threadPoolExecutor to optimize
 */
public class TextCrawler {

	private static final String TAG = TextCrawler.class.getSimpleName();
	private static final String KEY_HTML_TAG_HEAD_OPEN = "<head>";
	private static final String KEY_HTML_TAG_HEAD_CLOSE = "</head>";
	private static final String KEY_HTML_TAG_TITLE = "title";
	private static final String KEY_HTML_TAG_DESCRIPTION = "description";
	private static final String KEY_HTML_TAG_SITE_NAME = "site_name";
	private static final String KEY_EMPTY = "";
	private static final String KEY_HTML_TAG_IMAGE = "image";
	private static final String KEY_HTML_TAG_URL = "url";
	public static final int TIMEOUT_IN_MILLI = 1500;
	private final String HTTP_PROTOCOL = "http://";
	private final String HTTPS_PROTOCOL = "https://";
	private final String WWW_SUBDOMAIN = "www.";
	private LinkPreviewCallback mCallback;
	private SourceContent mSourceContent;
	private GetCode mGetCode;

	// We want at least 2 threads and at most 4 threads (if that's the max we have) in the core pool,
	// preferring to have 1 less than the CPU count to avoid saturating
	// the CPU with background work
	private static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

	// the number of threads to keep in the pool, even if they are idle, unless {@code allowCoreThreadTimeOut} is set
	private static final int CORE_POOL_SIZE = Math.max(2, Math.min(CPU_COUNT - 1, 4));

	// the maximum number of threads to allow in the pool
	private static final int MAXIMUM_POOL_SIZE = CPU_COUNT * 2 + 1;

	// keepAliveTime when the number of threads is greater than the core, this is the maximum time that excess idle threads will wait for new tasks before terminating.
	private static final int KEEP_ALIVE_SECONDS = 2;

	//
	private static final TimeUnit KEEP_ALIVE_TIME_UNIT = TimeUnit.SECONDS;
	private final BlockingQueue<Runnable> mDecodeWorkQueue  = new LinkedBlockingQueue<>();


	/**
	 * makePreview - initialize the url to parse and the callback to use with the result
	 */
	public void makePreview(LinkPreviewCallback callback, String url) {
		this.mCallback = callback;
		mSourceContent = new SourceContent();

		if (mGetCode != null) {
			mGetCode.cancel(true);
			mGetCode = null;
		}
		mGetCode = new GetCode();

		ThreadPoolExecutor mDecodeThreadPool = new ThreadPoolExecutor(
				CORE_POOL_SIZE,       // Initial pool size
				MAXIMUM_POOL_SIZE,    // Max pool size
				KEEP_ALIVE_SECONDS,
				KEEP_ALIVE_TIME_UNIT,
				mDecodeWorkQueue
		);
		mGetCode.executeOnExecutor(mDecodeThreadPool, url);
	}

	/**
	 * Get html code - inner class that extends AsyncTask
	 * to parse the relevant tags and return a result on the main thread
	 * it will return an error in case the parsing isn`t possible
	 */
	public class GetCode extends AsyncTask<String, Void, Void> {

		/**
		 * Finds urls inside the text and return the matched ones
		 */
		GetCode() { }

		@Override
		protected void onPreExecute() {
			if (mCallback != null) {
				mCallback.onPre();
			}
			super.onPreExecute();
		}

		@Override
		protected void onPostExecute(Void result) {
			if (mCallback != null) {
				mCallback.onPos(mSourceContent, isNull());
			}
			super.onPostExecute(result);
		}

		@Override
		protected Void doInBackground(String... params) {
			parseHtml(params);
			return null;
		}

		/**
		 * Verifies if the content could not be retrieved
		 */
		public boolean isNull() {
			return !mSourceContent.isSuccess() && extendedTrim(mSourceContent.getHtmlCode()).equals(KEY_EMPTY);
		}
	}

	private ArrayList<String> matches(String text, int results) {
		ArrayList<String> urls = new ArrayList<>();

		if (text == null) {
			LPMobileLog.w(TAG, "matches: given text is null");
			return urls;
		}

		text = text.replaceAll("\u200B", "");
		text = text.replaceAll("\u200C", "");
		text = text.replaceAll("\u200C", "");
		text = text.replaceAll("\uFEFF", "");

		String[] splitString = (text.split("\\s+"));
		for (String string : splitString) {

			// important - this code is to form a valid url to open web connection
			if (!string.startsWith(HTTP_PROTOCOL) && !string.startsWith(HTTPS_PROTOCOL) && Patterns.WEB_URL.matcher(string).matches()) {
				if (!string.startsWith(WWW_SUBDOMAIN)) {
					string = WWW_SUBDOMAIN + string;
				}
				string = HTTPS_PROTOCOL + string;
			}

			try {
				URL item = new URL(string);
				urls.add(item.toString());
			} catch (Exception e) {
				LPMobileLog.e(TAG, "ERROR", e);
			}
			if (results == 1 && urls.size() > 0)
				break;
		}

		return urls;
	}


	public void parseHtml(String[] params) {
		ArrayList<String> urls = matches(params[0], 0);
		if (mSourceContent == null) {
			mSourceContent = new SourceContent();
		}
		if (urls.size() <= 0) {
			mSourceContent.setFinalUrl("");

			// in case we have a valid url
		} else {
			mSourceContent.setFinalUrl((extendedTrim(urls.get(0))));
			StringBuilder myString = new StringBuilder();
			HttpURLConnection hc = null;

			try {
				String thisLine;
				URL u = new URL(mSourceContent.getFinalUrl());
				hc = (HttpURLConnection) u.openConnection();
				// setup timeout
				hc.setConnectTimeout(TIMEOUT_IN_MILLI);
				// This is IMPORTANT to prevent cases without og: tags just in mobile
				hc.setRequestProperty("User-Agent", "Mozilla");
				String charset = getCharsetEncoding(hc.getContentType());
				BufferedReader br = new BufferedReader(new InputStreamReader(hc.getInputStream(), charset));
				while ((thisLine = br.readLine()) != null) {
					myString.append(thisLine);
				}
				br.close();
				if (TextUtils.isEmpty(myString)) {
					throw new IOException("the url is empty");
				}

				// if the html is with extended tags
				if (myString.indexOf(KEY_HTML_TAG_HEAD_OPEN) == -1) {
					mSourceContent.setHtmlCode(extendedTrim(myString.toString()));
				} else {
					mSourceContent.setHtmlCode(extendedTrim(myString.substring(myString.indexOf(KEY_HTML_TAG_HEAD_OPEN),
							myString.lastIndexOf(KEY_HTML_TAG_HEAD_CLOSE) + KEY_HTML_TAG_HEAD_CLOSE.length())));
				}

				HashMap<String, String> metaTags = getMetaTags(mSourceContent.getHtmlCode());

				// save the meta tags for later use
				mSourceContent.setMetaTags(metaTags);
				mSourceContent.setTitle(metaTags.get(KEY_HTML_TAG_TITLE));
				mSourceContent.setDescription(metaTags.get(KEY_HTML_TAG_DESCRIPTION));
				mSourceContent.setSiteName(metaTags.get(KEY_HTML_TAG_SITE_NAME));

				// if we didn't found a title
				if (mSourceContent.getTitle().equals(KEY_EMPTY)) {
					String matchTitle = RegexUtil.pregMatch(mSourceContent.getHtmlCode(), RegexUtil.TITLE_PATTERN, 2);
					if (!matchTitle.equals(KEY_EMPTY)) {
						mSourceContent.setTitle((matchTitle));
					}
				}
				mSourceContent.setDescription(mSourceContent.getDescription().replaceAll(RegexUtil.SCRIPT_PATTERN, KEY_EMPTY));

				// if we didn't found an image
				if (!metaTags.get(KEY_HTML_TAG_IMAGE).equals(KEY_EMPTY)) {
					mSourceContent.setImages(metaTags.get(KEY_HTML_TAG_IMAGE));
				}
				mSourceContent.setSuccess(true);
			} catch (Exception e) {
				mSourceContent.setSuccess(false);
				mSourceContent.setUrl(KEY_EMPTY);
				mSourceContent.setHtmlCode(KEY_EMPTY);
				LPMobileLog.w(TAG, "url error: ", e);
			} finally {
				if (hc != null) {
					hc.disconnect();
				}
			}
		}

		// store the parsed data locally
		if (!TextUtils.isEmpty(mSourceContent.getFinalUrl())) {
			String[] finalLinkSet = mSourceContent.getFinalUrl().split("&");
			mSourceContent.setUrl(finalLinkSet[0]);
			mSourceContent.setCanonicalUrl(canonicalPage(mSourceContent.getFinalUrl()));
			mSourceContent.setDescription((mSourceContent.getDescription()));
			mSourceContent.setSiteName((mSourceContent.getSiteName()));
		}
	}

	@NonNull
	private String getCharsetEncoding(String contentType) {

		String charset = "";

		if (!TextUtils.isEmpty(contentType)) {
			String[] values = contentType.split(";");

			for (String value : values) {
				value = value.trim();

				if (value.toLowerCase().startsWith("charset=")) {
					charset = value.substring("charset=".length());
				}
			}
		}

		if ("".equals(charset)) {
            charset = "UTF-8";
        }
		return charset;
	}

	/**
	 * Returns the canonical url
	 */
	private String canonicalPage(String url) {
		String canonical = "";
		if (url.startsWith(HTTP_PROTOCOL)) {
			url = url.substring(HTTP_PROTOCOL.length());
		} else if (url.startsWith(HTTPS_PROTOCOL)) {
			url = url.substring(HTTPS_PROTOCOL.length());
		}

		int urlLength = url.length();
		for (int i = 0; i < urlLength; i++) {
			if (url.charAt(i) != '/') {
				canonical += url.charAt(i);
			} else {
				break;
			}
		}
		return canonical;
	}


	/**
	 * Verifies if the url is an image
	 */
	private boolean isImage(String url) {
		return !TextUtils.isEmpty(url) && url.matches(RegexUtil.IMAGE_PATTERN);
	}

	/**
	 * getMetaTags - extract the 4 tags we are using (image, url, title, description)
	 * using regex for open graph and non open graph tags
	 *
	 * @param content
	 * @return Returns meta tags from html code
	 */

	private HashMap<String, String> getMetaTags(String content) {
		HashMap<String, String> metaTags = new HashMap<>();
		metaTags.put(KEY_HTML_TAG_URL, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_TITLE, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_DESCRIPTION, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_IMAGE, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_SITE_NAME, KEY_EMPTY);

		List<String> matches = RegexUtil.pregMatchAll(content, RegexUtil.METATAG_PATTERN, 1);
		for (String match : matches) {
			final String lowerCase = match.toLowerCase();

			if (Configuration.getBoolean(R.bool.link_preview_to_use_more_than_og_tags)) {
				if (lowerCase.contains("name=\"url\"")
						|| lowerCase.contains("name='url'"))
					updateMetaTag(metaTags, "url", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"title\"")
						|| lowerCase.contains("name='title'"))
					updateMetaTag(metaTags, "title", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"description\"")
						|| lowerCase.contains("name='description'"))
					updateMetaTag(metaTags, "description", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"image\"")
						|| lowerCase.contains("name='image'"))
					updateMetaTag(metaTags, "image", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"site_name\"")
						|| lowerCase.contains("name='site_name'"))
					updateMetaTag(metaTags, "site_name", separateMetaTagsContent(match));
			}

			if (lowerCase.contains("property=\"og:url\"")
					|| lowerCase.contains("property='og:url'"))
				updateMetaTag(metaTags, "url", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:title\"")
					|| lowerCase.contains("property='og:title'"))
				updateMetaTag(metaTags, "title", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:description\"")
					|| lowerCase.contains("property='og:description'"))
				updateMetaTag(metaTags, "description", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:image\"")
					|| lowerCase.contains("property='og:image'"))
				updateMetaTag(metaTags, "image", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:site_name\"")
					|| lowerCase.contains("property='og:site_name'"))
				updateMetaTag(metaTags, "site_name", separateMetaTagsContent(match));


		}
		return metaTags;
	}

	private void updateMetaTag(HashMap<String, String> metaTags, String url, String value) {
		if (value != null && (value.length() > 0)) {
			metaTags.put(url, value);
		}
	}

	/**
	 * Gets content from MetaTag
	 *
	 * @param content
	 * @return
	 */
	private String separateMetaTagsContent(String content) {
		return RegexUtil.pregMatch(content, RegexUtil.METATAG_CONTENT_PATTERN, 1);
	}

	/**
	 * Removes extra spaces and trim the string
	 *
	 * @param content - the original string
	 * @return the new content
	 */
	static String extendedTrim(String content) {
		return TextUtils.isEmpty(content) ? "" : content.replaceAll("\\s+", " ").replace("\n", " ").replace("\r", " ").trim();
	}

	public SourceContent getSourceContent() {
		return mSourceContent;
	}
}

==> infra-4.4.0-sources.jar: com/liveperson/infra/utils/TextCrawler.java <==
package com.liveperson.infra.utils;

import android.os.AsyncTask;
import android.support.annotation.NonNull;
import android.support.annotation.VisibleForTesting;
import android.text.TextUtils;
import android.util.Patterns;

import com.liveperson.infra.R;
import com.liveperson.infra.configuration.Configuration;
import com.liveperson.infra.log.LPLog;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by maayank on 17/11/2016.
 * a util class for link preview feature (and for structure content)
 * receives a URL and parse the open graph (and other relevant) tags from it
 * it uses a callback to return the response
 * we use AsyncTask with threadPoolExecutor to optimize
 */
public class TextCrawler {

	private static final String TAG = "TextCrawler";
	private static final String KEY_HTML_TAG_HEAD_OPEN = "<head>";
	private static final String KEY_HTML_TAG_HEAD_CLOSE = "</head>";
	private static final String KEY_HTML_TAG_TITLE = "title";
	private static final String KEY_HTML_TAG_DESCRIPTION = "description";
	private static final String KEY_HTML_TAG_SITE_NAME = "site_name";
	private static final String KEY_EMPTY = "";
	private static final String KEY_HTML_TAG_IMAGE = "image";
	private static final String KEY_HTML_TAG_URL = "url";
	private static final int TIMEOUT_IN_MILLI = 1500;
	private static final String HTTP_PROTOCOL = "http://";
	private static final String HTTPS_PROTOCOL = "https://";
	private static final String WWW_SUBDOMAIN = "www.";

	private static final String TITLE_PATTERN = "<title(.*?)>(.*?)</title>";
	private static final String SCRIPT_PATTERN = "<script(.*?)>(.*?)</script>";
	private static final String METATAG_PATTERN = "<meta(.*?)>";
	private static final String METATAG_CONTENT_PATTERN = "content=\"(.*?)\"";

	private LinkPreviewCallback mCallback;
	private SourceContent mSourceContent;
	private GetCode mGetCode;

	// We want at least 2 threads and at most 4 threads (if that's the max we have) in the core pool,
	// preferring to have 1 less than the CPU count to avoid saturating
	// the CPU with background work
	private static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

	// the number of threads to keep in the pool, even if they are idle, unless {@code allowCoreThreadTimeOut} is set
	private static final int CORE_POOL_SIZE = Math.max(2, Math.min(CPU_COUNT - 1, 4));

	// the maximum number of threads to allow in the pool
	private static final int MAXIMUM_POOL_SIZE = CPU_COUNT * 2 + 1;

	// keepAliveTime when the number of threads is greater than the core, this is the maximum time that excess idle threads will wait for new tasks before terminating.
	private static final int KEEP_ALIVE_SECONDS = 2;

	//
	private static final TimeUnit KEEP_ALIVE_TIME_UNIT = TimeUnit.SECONDS;
	private final BlockingQueue<Runnable> mDecodeWorkQueue  = new LinkedBlockingQueue<>();


	/**
	 * makePreview - initialize the url to parse and the callback to use with the result
	 */
	public void makePreview(LinkPreviewCallback callback, String url) {
		this.mCallback = callback;
		mSourceContent = new SourceContent();

		if (mGetCode != null) {
			mGetCode.cancel(true);
			mGetCode = null;
		}
		mGetCode = new GetCode();

		ThreadPoolExecutor mDecodeThreadPool = new ThreadPoolExecutor(
				CORE_POOL_SIZE,       // Initial pool size
				MAXIMUM_POOL_SIZE,    // Max pool size
				KEEP_ALIVE_SECONDS,
				KEEP_ALIVE_TIME_UNIT,
				mDecodeWorkQueue
		);
		mGetCode.executeOnExecutor(mDecodeThreadPool, url);
	}

	/**
	 * Get html code - inner class that extends AsyncTask
	 * to parse the relevant tags and return a result on the main thread
	 * it will return an error in case the parsing isn`t possible
	 */
	public class GetCode extends AsyncTask<String, Void, Void> {

		/**
		 * Finds urls inside the text and return the matched ones
		 */
		GetCode() { }

		@Override
		protected void onPreExecute() {
			if (mCallback != null) {
				mCallback.onPre();
			}
			super.onPreExecute();
		}

		@Override
		protected void onPostExecute(Void result) {
			if (mCallback != null) {
				mCallback.onPos(mSourceContent, isNull());
			}
			super.onPostExecute(result);
		}

		@Override
		protected Void doInBackground(String... params) {
			parseHtml(params);
			return null;
		}

		/**
		 * Verifies if the content could not be retrieved
		 */
		public boolean isNull() {
			return !mSourceContent.isSuccess() && extendedTrim(mSourceContent.getHtmlCode()).equals(KEY_EMPTY);
		}
	}

	private ArrayList<String> matches(String text) {
		ArrayList<String> urls = new ArrayList<>();

		if (text == null) {
			LPLog.INSTANCE.w(TAG, "matches: given text is null");
			return urls;
		}

		text = text.replaceAll("\u200B", "");
		text = text.replaceAll("\u200C", "");
		text = text.replaceAll("\u200C", "");
		text = text.replaceAll("\uFEFF", "");

		String[] splitString = (text.split("\\s+"));
		for (String string : splitString) {

			// important - this code is to form a valid url to open web connection
			if (!string.startsWith(HTTP_PROTOCOL) && !string.startsWith(HTTPS_PROTOCOL) && Patterns.WEB_URL.matcher(string).matches()) {
				if (!string.startsWith(WWW_SUBDOMAIN)) {
					string = WWW_SUBDOMAIN + string;
				}
				string = HTTPS_PROTOCOL + string;
			}

			try {
				URL item = new URL(string);
				urls.add(item.toString());
			} catch (Exception e) {
				LPLog.INSTANCE.e(TAG, "ERROR", e);
			}
		}

		return urls;
	}


	public void parseHtml(String[] params) {
		ArrayList<String> urls = matches(params[0]);
		if (mSourceContent == null) {
			mSourceContent = new SourceContent();
		}
		if (urls.size() <= 0) {
			mSourceContent.setFinalUrl("");

			// in case we have a valid url
		} else {
			mSourceContent.setFinalUrl((extendedTrim(urls.get(0))));
			StringBuilder myString = new StringBuilder();
			HttpURLConnection hc = null;

			try {
				String thisLine;
				URL u = new URL(mSourceContent.getFinalUrl());
				hc = (HttpURLConnection) u.openConnection();
				// setup timeout
				hc.setConnectTimeout(TIMEOUT_IN_MILLI);
				// This is IMPORTANT to prevent cases without og: tags just in mobile
				hc.setRequestProperty("User-Agent", "Mozilla");
				String charset = getCharsetEncoding(hc.getContentType());
				BufferedReader br = new BufferedReader(new InputStreamReader(hc.getInputStream(), charset));
				while ((thisLine = br.readLine()) != null) {
					myString.append(thisLine);
				}
				br.close();
				if (TextUtils.isEmpty(myString)) {
					throw new IOException("the url is empty");
				}

				// if the html is with extended tags
				if (myString.indexOf(KEY_HTML_TAG_HEAD_OPEN) == -1) {
					mSourceContent.setHtmlCode(extendedTrim(myString.toString()));
				} else {
					mSourceContent.setHtmlCode(extendedTrim(myString.substring(myString.indexOf(KEY_HTML_TAG_HEAD_OPEN),
							myString.lastIndexOf(KEY_HTML_TAG_HEAD_CLOSE) + KEY_HTML_TAG_HEAD_CLOSE.length())));
				}

				HashMap<String, String> metaTags = getMetaTags(mSourceContent.getHtmlCode());

				// save the meta tags for later use
				mSourceContent.setMetaTags(metaTags);
				mSourceContent.setTitle(metaTags.get(KEY_HTML_TAG_TITLE));
				mSourceContent.setDescription(metaTags.get(KEY_HTML_TAG_DESCRIPTION));
				mSourceContent.setSiteName(metaTags.get(KEY_HTML_TAG_SITE_NAME));

				// if we didn't found a title
				if (mSourceContent.getTitle().equals(KEY_EMPTY)) {
					String matchTitle = pregMatch(mSourceContent.getHtmlCode(), TITLE_PATTERN, 2);
					if (!matchTitle.equals(KEY_EMPTY)) {
						mSourceContent.setTitle((matchTitle));
					}
				}
				mSourceContent.setDescription(mSourceContent.getDescription().replaceAll(SCRIPT_PATTERN, KEY_EMPTY));

				String tagImage = metaTags.get(KEY_HTML_TAG_IMAGE);
				// if we didn't found an image
				if (tagImage != null && !tagImage.equals(KEY_EMPTY)) {
					mSourceContent.setImages(metaTags.get(KEY_HTML_TAG_IMAGE));
				}
				mSourceContent.setSuccess(true);
			} catch (Exception e) {
				mSourceContent.setSuccess(false);
				mSourceContent.setUrl(KEY_EMPTY);
				mSourceContent.setHtmlCode(KEY_EMPTY);
				LPLog.INSTANCE.w(TAG, "url error: ", e);
			} finally {
				if (hc != null) {
					hc.disconnect();
				}
			}
		}

		// store the parsed data locally
		if (!TextUtils.isEmpty(mSourceContent.getFinalUrl())) {
			String[] finalLinkSet = mSourceContent.getFinalUrl().split("&");
			mSourceContent.setUrl(finalLinkSet[0]);
			mSourceContent.setCanonicalUrl(canonicalPage(mSourceContent.getFinalUrl()));
			mSourceContent.setDescription((mSourceContent.getDescription()));
			mSourceContent.setSiteName((mSourceContent.getSiteName()));
		}
	}

	@NonNull
	private String getCharsetEncoding(String contentType) {

		String charset = "";

		if (!TextUtils.isEmpty(contentType)) {
			String[] values = contentType.split(";");

			for (String value : values) {
				value = value.trim();

				if (value.toLowerCase().startsWith("charset=")) {
					charset = value.substring("charset=".length());
				}
			}
		}

		if ("".equals(charset)) {
            charset = "UTF-8";
        }
		return charset;
	}

	/**
	 * Returns the canonical url
	 */
	private String canonicalPage(String url) {
		String canonical = "";
		if (url.startsWith(HTTP_PROTOCOL)) {
			url = url.substring(HTTP_PROTOCOL.length());
		} else if (url.startsWith(HTTPS_PROTOCOL)) {
			url = url.substring(HTTPS_PROTOCOL.length());
		}

		int urlLength = url.length();
		for (int i = 0; i < urlLength; i++) {
			if (url.charAt(i) != '/') {
				canonical += url.charAt(i);
			} else {
				break;
			}
		}
		return canonical;
	}

	/**
	 * getMetaTags - extract the 4 tags we are using (image, url, title, description)
	 * using regex for open graph and non open graph tags
	 *
	 * @return Returns meta tags from html code
	 */

	private HashMap<String, String> getMetaTags(String content) {
		HashMap<String, String> metaTags = new HashMap<>();
		metaTags.put(KEY_HTML_TAG_URL, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_TITLE, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_DESCRIPTION, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_IMAGE, KEY_EMPTY);
		metaTags.put(KEY_HTML_TAG_SITE_NAME, KEY_EMPTY);

		List<String> matches = pregMatchMetaTagPattern(content);
		for (String match : matches) {
			final String lowerCase = match.toLowerCase();

			if (Configuration.getBoolean(R.bool.link_preview_to_use_more_than_og_tags)) {
				if (lowerCase.contains("name=\"url\"")
						|| lowerCase.contains("name='url'"))
					updateMetaTag(metaTags, "url", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"title\"")
						|| lowerCase.contains("name='title'"))
					updateMetaTag(metaTags, "title", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"description\"")
						|| lowerCase.contains("name='description'"))
					updateMetaTag(metaTags, "description", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"image\"")
						|| lowerCase.contains("name='image'"))
					updateMetaTag(metaTags, "image", separateMetaTagsContent(match));
				else if (lowerCase.contains("name=\"site_name\"")
						|| lowerCase.contains("name='site_name'"))
					updateMetaTag(metaTags, "site_name", separateMetaTagsContent(match));
			}

			if (lowerCase.contains("property=\"og:url\"")
					|| lowerCase.contains("property='og:url'"))
				updateMetaTag(metaTags, "url", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:title\"")
					|| lowerCase.contains("property='og:title'"))
				updateMetaTag(metaTags, "title", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:description\"")
					|| lowerCase.contains("property='og:description'"))
				updateMetaTag(metaTags, "description", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:image\"")
					|| lowerCase.contains("property='og:image'"))
				updateMetaTag(metaTags, "image", separateMetaTagsContent(match));
			else if (lowerCase.contains("property=\"og:site_name\"")
					|| lowerCase.contains("property='og:site_name'"))
				updateMetaTag(metaTags, "site_name", separateMetaTagsContent(match));


		}
		return metaTags;
	}

	private void updateMetaTag(HashMap<String, String> metaTags, String url, String value) {
		if (value != null && (value.length() > 0)) {
			metaTags.put(url, value);
		}
	}

	/**
	 * Gets content from MetaTag
	 */
	private String separateMetaTagsContent(String content) {
		return pregMatch(content, METATAG_CONTENT_PATTERN, 1);
	}

	/**
	 * Removes extra spaces and trim the string
	 *
	 * @param content - the original string
	 * @return the new content
	 */
	private String extendedTrim(String content) {
		return TextUtils.isEmpty(content) ? "" : content.replaceAll("\\s+", " ").replace("\n", " ").replace("\r", " ").trim();
	}

	@VisibleForTesting
	public SourceContent getSourceContent() {
		return mSourceContent;
	}

	private String pregMatch(String content, String pattern, int index) {
		String match = "";
		Matcher matcher = Pattern.compile(pattern).matcher(content);

		if (matcher.find()) {
			match = matcher.group(index);
		}
		return extendedTrim(match);
	}

	private List<String> pregMatchMetaTagPattern(String content) {
		List<String> matches = new ArrayList<>();
		Matcher matcher = Pattern.compile(METATAG_PATTERN).matcher(content);

		while (matcher.find()) {
			matches.add(extendedTrim(matcher.group(1)));
		}
		return matches;
	}
}
